/******************************************************************************
 *{@C
 *      Copyright:      2009-2012 Paul Obermeier (obermeier@tcl3d.org)
 *
 *                      See the file "Tcl3D_License.txt" for information on
 *                      usage and redistribution of this file, and for a
 *                      DISCLAIMER OF ALL WARRANTIES.
 *
 *      Module:         Tcl3D -> tcl3dOsg
 *      Filename:       osgUtil.i
 *
 *      Author:         Paul Obermeier
 *
 *      Description:    SWIG file for wrapping the Open Scene Graph library.
 *                      This file does the actual wrapping of OSG module 
 *                      osgUtil.
 *
 *****************************************************************************/

%include "osgHeader.i"

%include "osgAnimationHeader.i"
%include "osgParticleHeader.i"
%include "osgSimHeader.i"
%include "osgTextHeader.i"
%include "osgUtilHeader.i"

%import "osg.i"

%{

using namespace osg;
using namespace osgUtil;

%}

%ignore osgUtil::SceneView::ComputeStereoMatricesCallback;
%ignore osgUtil::SceneView::setComputeStereoMatricesCallback;
%ignore osgUtil::SceneView::getComputeStereoMatricesCallback;

%ignore osgUtil::RenderBin::setSortCallback;
%ignore osgUtil::RenderBin::getSortCallback;
%ignore osgUtil::RenderBin::setDrawCallback;
%ignore osgUtil::RenderBin::getDrawCallback;

%ignore osgUtil::Optimizer::setIsOperationPermissibleForObjectCallback;
%ignore osgUtil::Optimizer::getIsOperationPermissibleForObjectCallback;
%ignore osgUtil::Simplifier::setContinueSimplificationCallback;
%ignore osgUtil::Simplifier::getContinueSimplificationCallback;
%ignore osgUtil::Simplifier::simplify;

%rename(get) osgUtil::LessDepthSortFunctor::operator ();

%feature("notabstract") osgUtil::SceneView;

%include osgUtil/RenderLeaf
%include osgUtil/StateGraph
%include osgUtil/RenderBin
%include osgUtil/PositionalStateContainer
%include osgUtil/RenderStage
%include osgUtil/CubeMapGenerator

%ignore osgUtil::CullVisitor::getIdentifier;
%ignore osgUtil::CullVisitor::setIdentifier;
%include osgUtil/CullVisitor

%include osgUtil/DelaunayTriangulator
%include osgUtil/DisplayRequirementsVisitor
%include osgUtil/GLObjectsVisitor
%include osgUtil/HalfWayMapGenerator
%include osgUtil/HighlightMapGenerator

// Duplicate nested class from osgUtil/IntersectionVisitor
struct ReadCallback : public osg::Referenced
{
    virtual osg::Node* readNodeFile(const std::string& filename) = 0;
};
%include osgUtil/IntersectionVisitor
%{
    typedef osgUtil::IntersectionVisitor::ReadCallback ReadCallback;
%}

%include osgUtil/IntersectVisitor

%include osgUtil/Optimizer
%include osgUtil/ReflectionMapGenerator
%include osgUtil/SceneView
%include osgUtil/Simplifier
%include osgUtil/SmoothingVisitor
%include osgUtil/Statistics
%include osgUtil/TangentSpaceGenerator
%include osgUtil/TransformAttributeFunctor
%include osgUtil/TransformCallback
%include osgUtil/TriStripVisitor
%include osgUtil/UpdateVisitor

// Duplicate nested class from osgUtil/LineSegmentIntersector.
// CHECK when updating to a newer OSG version.
struct Intersection
{
    Intersection():
        ratio(-1.0),
        primitiveIndex(0) {}

    bool operator < (const Intersection& rhs) const { return ratio < rhs.ratio; }

    typedef std::vector<unsigned int>   IndexList;
    typedef std::vector<double>         RatioList;

    double                          ratio;
    osg::NodePath                   nodePath;
    osg::ref_ptr<osg::Drawable>     drawable;
    osg::ref_ptr<osg::RefMatrix>    matrix;
    osg::Vec3d                      localIntersectionPoint;
    osg::Vec3                       localIntersectionNormal;
    IndexList                       indexList;
    RatioList                       ratioList;
    unsigned int                    primitiveIndex;

    const osg::Vec3d& getLocalIntersectPoint() const {
        return localIntersectionPoint; 
    }
    osg::Vec3d getWorldIntersectPoint() const {
        return matrix.valid() ? localIntersectionPoint * (*matrix) : localIntersectionPoint; 
    }

    const osg::Vec3& getLocalIntersectNormal() const {
        return localIntersectionNormal; 
    }
    osg::Vec3 getWorldIntersectNormal() const {
        return matrix.valid() ?
            osg::Matrix::transform3x3(osg::Matrix::inverse(*matrix),localIntersectionNormal) :
            localIntersectionNormal; 
    }
};

%nestedworkaround osgUtil::LineSegmentIntersector::Intersection;

%typemap(out) osgUtil::LineSegmentIntersector::Intersections& 
{
    for (osgUtil::LineSegmentIntersector::Intersections::iterator i = $1->begin();
         i != $1->end(); ++i) {
        Tcl_Obj *obj = SWIG_NewPointerObj((new Intersection(static_cast< const Intersection& >(*i))), SWIGTYPE_p_Intersection, SWIG_POINTER_OWN |  0 );

        if (obj) {
            if (Tcl_ListObjAppendElement (interp, $result, obj) == TCL_ERROR) {
                return NULL;
            }
        }
    }
}

%include osgUtil/LineSegmentIntersector

%{
    typedef osgUtil::LineSegmentIntersector::Intersection Intersection;
%}

%include osgUtil/Version
